import pandas as pd
from sklearn.preprocessing import StandardScaler, normalize
import numpy as np

class FeatureScaler:
    '''This class is designed to allow you to scale your data with ease. You have the ability to scaler your
by using StandardScaler or normalize. Not only that, but you can also scale either the whole dataset or
you can just scale one column at a time.
the initialization of this module is
FeatureScaler(data,scaler=StandardScaler,columns=[])
data is where you can either enter a pandas data frame or a NumPy array,
by default the scaler this module will use is StandardScaler, but you can change this variable to normalize to use
normalization.
the columns input argument is to allow you the ability to have more control over what columns get scaled if you leave this
empty, you will end up scaling the whole dataset
to tell the module to do something with the data, all you have to do is call the check method by .check()
this will return the scaled data that you imported in the same format that you entered it as

'''
    def __init__(self,data, scaler = "StandardScaler", columns=[]):
        self.data = data
        self.scaler = scaler
        self.columns = columns
        if isinstance(data, np.ndarray):
            self.type = "numpy"
        elif isinstance(data, pd.core.frame.DataFrame):
            self.type = "pandas"

    def check(self):
        if self.scaler.lower() == "standardscaler":
            self.standardscaler_()
        elif self.scaler.lower() =="normalize":
            self.normalizion_()
        if self.type == "pandas":
            return pd.DataFrame(data = self.data,columns= self.columns_names)
        else:

            return self.data
    def standardscaler_(self):
        scaler = StandardScaler()
        if self.type == "pandas":
            self.converting_to_np_()

        if len(self.columns)>0:
            for  i in  self.columns:
                self.data[:,i] = scaler.fit_transform(self.data[:,i].reshape(-1,1)).flatten()

        else:
            for i in range(self.data.shape[1]):
                self.data[:,i] = scaler.fit_transform(self.data[:, i].reshape(-1,1)).flatten()
    def normalizion_(self):
        if self.type == "pandas":
            self.converting_to_np_()
        if len(self.columns)>0:
            for i in self.columns:
                self.data[:,i] = normalize(self.data[:,i].reshape(-1,1)).flatten()


        else:
            for i in range(self.data.shape[1]):
                self.data[:,i] = normalize(self.data[:,i].reshape(-1,1)).flatten()

    def converting_to_np_(self):
        self.copy = self.data
        self.columns_names = self.data.columns
        temp = np.array(self.data.iloc[:,:])
        self.data = temp
        if len(self.columns)>0:
            column_dt = {}
            index = []
            for i in range(len(self.columns_names)):
                column_dt[self.columns_names[i]] = i
            for i in self.columns:
                index.append(column_dt[i])
            self.columns = index
if __name__ == "__main__":
    import islanders as ir
    data = ir.datasets(
        "titanic")
    scaler = FeatureScaler(data = data, scaler = "normalize")
    scaler.check()


import pandas as pd
from sklearn.preprocessing import OrdinalEncoder
from .stringtodatetime import StringToDateTime
from .inconsistent_data import InconsistentData
import numpy as np
class Encoder:
    def __init__(self,df,type = "", columns = []):
        ''' this class is dessigned to help you make encoding your data simple
        the input variables for this class are
        df: a pandas dataframe
        type: by defalult this variable will br set to ONEHOTENCODER if you with to use
        OrdinalEncoder you would set type to ordinalencoder
        columns: when you have specific columns you want to be be corrected a specific way
        then import the column or column names to this variable and only those columns will get corrected
        they type you specificed
        then you can call the check method to make the corretions
        this method will return a pandas data frame
        if you wish to compare the returned value to the original dataset you may
        call copy'''
        self.df = df
        self.copy = df.copy()
        self.type = type
        self.columns = columns
        datetime = StringToDateTime(self.df)
        self.df = datetime.check()
        correcting = InconsistentData(self.df)
        correcting.column_names_white_space()
        self.df = correcting.data_white_space()
        self.np = np.array(self.df.iloc[:,:].values)
        self.categories = []
    def check(self):
        if (len(self.columns)==0):
            self.object_column = []
            for i in self.df.columns:
                if (self.df[i].dtype == "object"):
                    self.object_column.append(i)
        else:
            # print("yes")

            self.object_column = [i for i in self.columns]
        self.Correct()
        # print(self.object_column)
        # print("yes")
        # print(self.df)
        return self.df
    def Correct(self):
        if (self.type == "" or self.type.upper() == "ONEHOTENCODER"):
            self.OneHotEncoder()
        elif (self.type.upper() == "ORDINALENCODER"):
            self.OrdinalEncoder()
    def OrdinalEncoder(self):
        columns = [i for i in self.df.columns]
        for i in self.object_column:
            for j in range(len(self.df.columns)):
                if (columns[j] == i):
                    # print(self.df[i].array.reshape(-1,1))
                    self.categories.append(self.df[i].value_counts().keys())
                    translate = OrdinalEncoder()
                    final = translate.fit_transform(self.df[i].array.reshape(-1,1))
                    self.df.drop(columns=i, inplace=True)
                    self.df[i] = final
        # print(self.categories)
    def OneHotEncoder(self):
        for i in self.object_column:
            new_df = pd.get_dummies(self.df[i])
            for  j in new_df.columns:
                self.df[j] = new_df[j]
        self.df.drop(columns=self.object_column,inplace=True)




if __name__ == "__main__":
    import pandas as pd
    import numpy as np
    import islanders as ir
    import irdatacleaning
    from tensorflow import keras
    data = ir.datasets("amazon electronics")
    name = [i for i in data.name]
    data.drop(columns = "name", inplace=True)
    encoder = Encoder(df = data, type="ordinalencoder")
    data = encoder.check()
    print(encoder.categories)
